from Screens.Screen import Screen
from Screens.Console import Console
from Screens.MessageBox import MessageBox
from Components.MenuList import MenuList
from Components.ActionMap import ActionMap
from Components.config import config, ConfigSubsection, ConfigSelection, getConfigListEntry, ConfigYesNo, ConfigDirectory
from Components.ConfigList import ConfigListScreen
from Components.FileList import FileList
from Components.Label import Label
from Components.Language import language
from Components.Sources.List import List
from Plugins.Plugin import PluginDescriptor
from Tools.Directories import fileExists, pathExists, resolveFilename, SCOPE_PLUGINS, SCOPE_LANGUAGE, SCOPE_MEDIA
import gettext
from os import environ
import os
from enigma import getDesktop, quitMainloop

lang = language.getLanguage()
environ["LANGUAGE"] = lang[:2]
gettext.bindtextdomain("enigma2", resolveFilename(SCOPE_LANGUAGE))
gettext.textdomain("enigma2")
gettext.bindtextdomain("ImageManager", "%s%s" % (resolveFilename(SCOPE_PLUGINS), "Extensions/ImageManager/locale/"))

def _(txt):
    t = gettext.dgettext("ImageManager", txt)
    if t == txt:
        t = gettext.gettext(txt)
    return t

skin_hd_main = """
  <screen name="ImageManager" position="center,130" size="1100,500" >
    <ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/ImageManager/icon/background.png" position="850,50" size="230,400" zPosition="2" alphatest="on" />
    <widget name="config" position="50,50" size="750,200" transparent="1" scrollbarMode="showOnDemand" zPosition="2" />
    <widget render="Label" source="key_red" position="100,250" size="400,40" transparent="1" zPosition="5" valign="center" halign="left" font="Regular;21" foregroundColor="#00FF1800" />
    <widget render="Label" source="key_green" position="100,300" size="400,40" transparent="1" zPosition="5" valign="center" halign="left" font="Regular;21" foregroundColor="#0000FF30" />
    <widget render="Label" source="key_yellow" position="100,350" size="400,40" transparent="1" zPosition="5" valign="center" halign="left" font="Regular;21" foregroundColor="#00F0FF00" />
    <widget render="Label" source="key_blue" position="100,400" size="400,40" transparent="1" zPosition="5" valign="center" halign="left" font="Regular;21" foregroundColor="#00004EFF" />
  </screen>"""
skin_sd_main = """
  <screen name="ImageManager" position="center,110" size="600,400" >
    <widget name="config" position="20,40" size="560,150" transparent="1" scrollbarMode="showOnDemand" zPosition="2" />
    <widget render="Label" source="key_red" position="30,220" size="500,40" transparent="1" zPosition="5" valign="center" halign="left" font="Regular;21" foregroundColor="#00FF1800" />
    <widget render="Label" source="key_green" position="30,260" size="500,40"  transparent="1" zPosition="5" valign="center" halign="left" font="Regular;21" foregroundColor="#0000FF30" />
    <widget render="Label" source="key_yellow" position="30,300" size="500,40" transparent="1" zPosition="5" valign="center" halign="left" font="Regular;21" foregroundColor="#00F0FF00" />
    <widget render="Label" source="key_blue" position="30,340" size="500,40" transparent="1" zPosition="5" valign="center" halign="left" font="Regular;21" foregroundColor="#00004EFF" />
  </screen>"""

skin_hd_filelist = """
  <screen position="center,center" size="700,600" title="Image Installer" >
    <widget name="filelist" position="10,10" size="680,580" foregroundColor="#0000FF80" scrollbarMode="showOnDemand" />
  </screen>"""
skin_sd_filelist = """
  <screen position="center,center" size="550,400" title="Image Installer" >
    <widget name="filelist" position="10,10" size="530,380" foregroundColor="#0000FF80" scrollbarMode="showOnDemand" />
  </screen>"""

class ImageManager(ConfigListScreen, Screen):
    def __init__(self, session):
        self.session = session
        Screen.__init__(self, session)
        if getDesktop(0).size().width() == 1280:
            self.skin = skin_hd_main
        else:
            self.skin = skin_sd_main

        def getMountedDevs(aaa):
            mountedDevs = []
            mountedDevs.append(('NAND /dev/mtdblock6', 'NAND-Flash'))
            a=os.popen("blkid -c /dev/null -w /dev/null | grep 'ext'")
            for b in a.readlines():
                DevList = b.strip('\n')
                if DevList.find('TYPE="ext') > 0:
                    SDXY = DevList.split(":",1)[0]
                    if DevList.find('LABEL="') > 0:
                        NAME = DevList[DevList.find('LABEL="')+7:]
                        NAME = NAME[:NAME.find('"')]
                    else:
                        NAME = SDXY[SDXY.find('/dev/')+5:]
                        NAME = NAME.upper()
                    mountedDevs.append((NAME+' '+SDXY, NAME+_(' on ')+SDXY))
            if aaa == 2:
                a=open("/proc/mounts", "r")
                for b in a.readlines():
                    DevList = b.strip('\n')
                    if DevList.find(' vfat ') > -1:
                        NAME = DevList[DevList.find('/media/')+7:]
                        NAME = NAME[:NAME.find(' ')]
                        SDXY = DevList[:DevList.find(' ')]
                        mountedDevs.append((NAME+' '+SDXY, NAME+_(' on ')+SDXY))
                    if DevList.find('/media/net/') > -1:
                        NAMEshort = DevList[DevList.find('/media/')+11:]
                        NAMEshort = NAMEshort[:NAMEshort.find(' ')]
                        NAMEfull = DevList[DevList.find('/media/'):]
                        NAMEfull = NAMEfull[:NAMEfull.find(' ')]
                        SDXY = DevList[:DevList.find(' ')]
                        mountedDevs.append((NAMEfull+' '+NAMEfull, NAMEshort+_(' on ')+SDXY))
                a.close()
            return mountedDevs

        config.plugins.ImageManager = ConfigSubsection()
        config.plugins.ImageManager.workdir = ConfigDirectory()
        config.plugins.ImageManager.devsFrom = ConfigSelection(choices=getMountedDevs(1))
        config.plugins.ImageManager.devsTo = ConfigSelection(choices=getMountedDevs(2))
        config.plugins.ImageManager.imagetype = ConfigYesNo(default = False)
        config.plugins.ImageManager.archivetype = ConfigSelection(default="IMG", choices = [
                   ("IMG", "IMG"),
                   ("TAR", "TAR"),
                   ("TARGZ", "TAR.GZ")])
        config.plugins.ImageManager.emu = ConfigSelection(default="XXX", choices = [
                   ("WMO", "Wicardd, MgCamd, Oscam"),
                   ("WM", "Wicardd, MgCamd"),
                   ("WO", "Wicardd, Oscam"),
                   ("MO", "MgCamd, Oscam"),
                   ("O", "Oscam"),
                   ("W", "Wicardd"),
                   ("M", "MgCamd"),
                   ("XXX", _("no"))])

        self.list = []
        self.list.append(getConfigListEntry(_("Select the source partition:"), config.plugins.ImageManager.devsFrom))
        self.list.append(getConfigListEntry(_("Select the target partition:"), config.plugins.ImageManager.devsTo))
        self.list.append(getConfigListEntry(_("Select the type of archive:"), config.plugins.ImageManager.archivetype))
        self.list.append(getConfigListEntry(_("Delete the configuration file Enigma?"), config.plugins.ImageManager.imagetype))
        self.list.append(getConfigListEntry(_("Remove config settings EMU?"), config.plugins.ImageManager.emu))
        ConfigListScreen.__init__(self, self.list)
        self.setTitle(_("Image Manager ver 1.8.1 (c)Vasiliks"))
        self['key_red'] = Label(_('Install to the target partition'))
        self['key_green'] = Label(_('Backup to the target partition'))
        self['key_yellow'] = Label(_('Reboot from the source partition'))
        self['key_blue'] = Label(_('Copying from partition to partition'))
        self['myActionMap'] = ActionMap(['ColorActions', 'OkCancelActions', 'StandbyActions'], { 'ok': self.activepart,
           'red': self.install,
           'power':self.reboot_spark,
           'green': self.makebackup,
           'yellow': self.reboot,
           'blue': self.copying,
           'cancel': self.cancel}, -2)

    def cancel(self):
        self.close(None)

    def activepart(self):
        f = open("/proc/cmdline", "r")
        b = f.readline()
        f.close()
        c = b.strip('\n')
        activepart = c[c.find('/dev/'):]
        activepart = activepart[:activepart.find(' ')]
        self.message = _("Active Partition  -  ") + activepart
        self.session.open(MessageBox, self.message, type = MessageBox.TYPE_INFO, timeout = 7 )
        return

    def copying(self):
        self.error = ""
        if config.plugins.ImageManager.devsTo.value == config.plugins.ImageManager.devsFrom.value:
            self.error =  _("You cannot copy a partition to itself!")
        c = config.plugins.ImageManager.devsTo.value[config.plugins.ImageManager.devsTo.value.find('/dev/'):]
        f = open("/proc/cmdline", "r")
        b = f.readline()
        f.close()
        if b.find(c) > 0:
            self.error = _("You cannot copy the active partition!")
        if config.plugins.ImageManager.devsTo.value.find("HDD") >-1 or config.plugins.ImageManager.devsTo.value.find(":/") > -1:
            self.error = _("You cannot copy this partition!")
        if self.error != "":
            self.session.open(MessageBox, self.error, type = MessageBox.TYPE_INFO, timeout = 5 )
            return
        self.makeCopy = '/usr/lib/enigma2/python/Plugins/Extensions/ImageManager/bin/copying.sh'
        self.makeCopy += ' %s %s %s' % (config.plugins.ImageManager.devsFrom.value, config.plugins.ImageManager.devsTo.value, config.plugins.ImageManager.imagetype.value)
        self.session.openWithCallback(self.Yes, Console,  _('Copying of partition'), ['%s' % self.makeCopy])
        return

    def Yes(self):
        self.session.openWithCallback(self.rebootCopy, MessageBox,_("Do you want to reboot with the new partition?"), timeout = 0, default = False)

    def rebootCopy(self, answer):
        if answer:
            devBoot = config.plugins.ImageManager.devsTo.value[config.plugins.ImageManager.devsTo.value.find('/dev/'):]
            self.reBootAll(devBoot)

    def reboot(self):
        devBoot = config.plugins.ImageManager.devsFrom.value[config.plugins.ImageManager.devsFrom.value.find('/dev/'):]
        self.reBootAll(devBoot)

    def reboot_spark(self):
        devBoot = 'SPARK'
        self.reBootAll(devBoot)

    def reBootAll(self, devBoot):
        self.session.nav.stopService()
        self.MBoot = '/usr/lib/enigma2/python/Plugins/Extensions/ImageManager/bin/setIMG.sh'
        self.param = ' NAND'
        if devBoot == 'SPARK':
            self.param = ' SPARK'
        elif devBoot != '/dev/mtdblock6':
            self.param = ' USB ' + devBoot
        os.system(self.MBoot + self.param)
        if fileExists('/tmp/mboot.log'):
            file = open('/tmp/mboot.log', "r")
            lines = file.readlines()
            file.close()
            self.session.open(MessageBox, ''.join(lines), type = MessageBox.TYPE_INFO, timeout = 10 )
        if fileExists('/tmp/mboot.ok'):
            quitMainloop(2)
        elif fileExists('/tmp/mboot.error'):
            pass
        else:
            self.session.open(MessageBox, "Unknown error occured :(", type = MessageBox.TYPE_INFO, timeout = 5 )
            return

    def makebackup(self):
        if config.plugins.ImageManager.devsTo.value[0:6] == 'NAND /':
            self.session.open(MessageBox, _("You cannot make backup to NAND!"), type = MessageBox.TYPE_INFO, timeout = 5 )
            return
        self.makeBackup = '/usr/lib/enigma2/python/Plugins/Extensions/ImageManager/bin/backup.sh'
        self.makeBackup += ' %s %s %s %s %s' % (config.plugins.ImageManager.devsFrom.value, config.plugins.ImageManager.devsTo.value, config.plugins.ImageManager.archivetype.value, config.plugins.ImageManager.imagetype.value, config.plugins.ImageManager.emu.value)
        self.session.open(Console, _('Backup Creator'), ['%s' % self.makeBackup])

    def install(self):
        self.error = ""
        c = config.plugins.ImageManager.devsTo.value[config.plugins.ImageManager.devsTo.value.find('/dev/'):]
        f = open("/proc/cmdline", "r")
        b = f.readline()
        f.close()
        if b.find(c) > 0:
            self.error = _("You cannot install the active partition!")
        if config.plugins.ImageManager.devsTo.value.find("HDD") >-1 or config.plugins.ImageManager.devsTo.value.find(":/") > -1:
            self.error = _("You cannot install this partition!")
        if self.error != "":
            self.session.open(MessageBox, self.error, type = MessageBox.TYPE_INFO, timeout = 5 )
            return
        self.session.open(Install)

class Install(Screen):
    def __init__(self, session, args = None):
        Screen.__init__(self, session)
        if getDesktop(0).size().width() == 1280:
            self.skin = skin_hd_filelist
        else:
            self.skin = skin_sd_filelist
        self.setTitle(_("Select the image"))
        self.session = session
        self.workdir = config.plugins.ImageManager.workdir.getValue()
        hide = ['/bin', '/boot', '/dev', '/dev.static', '/etc', '/lib', '/proc', '/ram', '/root', '/sbin', '/sys', '/tmp', '/usr', '/var']
        self["filelist"] = FileList(self.workdir, showMountpoints = True, matchingPattern = "(?i)^.*\.(img|tar|gz)", inhibitDirs=hide, )
        self["actions"] = ActionMap(["WizardActions"],
        {   "ok": self.ok,
            "back": self.back,
            "up": self.up,
            "down": self.down,
            "left": self.left,
            "right": self.right, }, -1)

    def back(self):
        self.close()

    def up(self):
        self["filelist"].up()

    def down(self):
        self["filelist"].down()

    def left(self):
        self["filelist"].pageUp()

    def right(self):
        self["filelist"].pageDown()

    def ok(self):
        if self["filelist"].canDescent():
            self["filelist"].descent()
        else:
            if self["filelist"].getFilename() != None:
                self.workdir = self["filelist"].getCurrentDirectory()
                self.install = '/usr/lib/enigma2/python/Plugins/Extensions/ImageManager/bin/install.sh'
                self.install += ' %s %s %s' % (config.plugins.ImageManager.devsTo.value[config.plugins.ImageManager.devsTo.value.find('/dev/'):], self["filelist"].getCurrentDirectory(), self["filelist"].getFilename())
                self.session.openWithCallback(self.Execution, MessageBox,_("Do you want to install with this image?"), timeout = 0, default = True)

    def Execution(self, answer):
        if answer:
            self.session.openWithCallback(self.close, Console,  _('Image Installer'), ['%s' % self.install])

def start(session, **kwargs):
    session.open(ImageManager)

def main(session, **kwargs):
#    global SparkReboot
    SparkReboot = ImageManager(session)
    SparkReboot.reboot_spark()

def startSetup(menuid):
    if menuid == "shutdown":
        return [(_("Reboot to Spark"), main, "reboot_to_spark", None)]
    else:
        return []

def Plugins(**kwargs):
    return [PluginDescriptor(where = PluginDescriptor.WHERE_MENU, fnc = startSetup),
            PluginDescriptor(name=_('Image Manager'), description = _("manage of your image Enigma"),
            where = [PluginDescriptor.WHERE_PLUGINMENU,  PluginDescriptor.WHERE_EXTENSIONSMENU],
            icon="icon/ImageManager.png", fnc=start),]

